@@ -1,4 +1,4 @@
-csetMV <- function(dat, n, method, alpha=0.1, scale="var", steps=400){
+csetMV <- function(dat, n, method, alpha=0.1, scale="var", steps=500){
   
   Var1 <- Var2 <- NULL # just to appease RCMD check
   
@@ -62,12 +62,13 @@ csetMV <- function(dat, n, method, alpha=0.1, scale="var", steps=400){
         searchwidth <- searchwidth + 1
         
       }
-      
+    
     }
     
     if(method=="large"){
       
-      grid[, 3] <- n/grid[, 2] * (mea - grid[, 1])^2 + n/(2 * grid[, 2]^2) * (s^2 - grid[, 2])^2 < qchisq(1 - alpha, df=2)
+      rhs <- qchisq(1 - alpha, df=2)
+      grid[, 3] <- n/grid[, 2] * (mea - grid[, 1])^2 + n/(2 * grid[, 2]^2) * (s^2 - grid[, 2])^2 < rhs
       
       crFinal <- grid[grid[, 3]==TRUE, ]
       ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
@@ -92,11 +93,54 @@ csetMV <- function(dat, n, method, alpha=0.1, scale="var", steps=400){
         
       }
       
+      Msteps <- Vsteps <- steps
+      
+      while((nrow(crFinal[crFinal$Var1==min(crFinal$Var1), ]) > 5 |
+              nrow(crFinal[crFinal$Var1==max(crFinal$Var1), ]) > 5) & Msteps < 2001){
+        
+        Msteps <- 2 * Msteps
+        
+        togrid <- list()
+        togrid[[1]] <- seq(mea - searchwidth * qnorm(1 - alpha/16) * s / sqrt(n),
+                           mea + searchwidth * qnorm(1 - alpha/16) * s / sqrt(n), length.out=Msteps)
+        togrid[[2]] <- seq(s^2 * 1/searchwidth * n / qchisq(df=df, 1 - alpha/16),
+                           s^2 * searchwidth * n / qchisq(df=df, alpha/16), length.out=steps)
+        
+        grid <- expand.grid(togrid)
+        
+        grid[, 3] <- n/grid[, 2] * (mea - grid[, 1])^2 + n/(2 * grid[, 2]^2) * (s^2 - grid[, 2])^2 < rhs
+        
+        crFinal <- grid[grid[, 3]==TRUE, ]
+        ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
+                
+      }
+      
+      while((nrow(crFinal[crFinal$Var2==min(crFinal$Var2), ]) > 10 |
+              nrow(crFinal[crFinal$Var2==max(crFinal$Var2), ]) > 10) & Vsteps < 2001){
+        
+        Vsteps <- 2 * Vsteps
+        
+        togrid <- list()
+        togrid[[1]] <- seq(mea - searchwidth * qnorm(1 - alpha/16) * s / sqrt(n),
+                           mea + searchwidth * qnorm(1 - alpha/16) * s / sqrt(n), length.out=Msteps)
+        togrid[[2]] <- seq(s^2 * 1/searchwidth * n / qchisq(df=df, 1 - alpha/16),
+                           s^2 * searchwidth * n / qchisq(df=df, alpha/16), length.out=Vsteps)
+        
+        grid <- expand.grid(togrid)
+        
+        grid[, 3] <- n/grid[, 2] * (mea - grid[, 1])^2 + n/(2 * grid[, 2]^2) * (s^2 - grid[, 2])^2 < rhs
+        
+        crFinal <- grid[grid[, 3]==TRUE, ]
+        ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
+        
+      }
+      
     }
     
     if(method=="plugin"){
       
-      grid[, 3] <- n/s^2 * (mea - grid[, 1])^2 + n/(2 * s^4) * (s^2 - grid[, 2])^2 < qchisq(1 - alpha, df=2)
+      rhs <- qchisq(1 - alpha, df=2)
+      grid[, 3] <- n/s^2 * (mea - grid[, 1])^2 + n/(2 * s^4) * (s^2 - grid[, 2])^2 < rhs
       
       crFinal <- grid[grid[, 3]==TRUE, ]
       ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
@@ -121,11 +165,54 @@ csetMV <- function(dat, n, method, alpha=0.1, scale="var", steps=400){
         
       }
       
+      Msteps <- Vsteps <- steps
+      
+      while((nrow(crFinal[crFinal$Var1==min(crFinal$Var1), ]) > 5 |
+              nrow(crFinal[crFinal$Var1==max(crFinal$Var1), ]) > 5) & Msteps < 2001){
+        
+        Msteps <- 2 * Msteps
+        
+        togrid <- list()
+        togrid[[1]] <- seq(mea - searchwidth * qnorm(1 - alpha/16) * s / sqrt(n),
+                           mea + searchwidth * qnorm(1 - alpha/16) * s / sqrt(n), length.out=Msteps)
+        togrid[[2]] <- seq(s^2 * 1/searchwidth * n / qchisq(df=df, 1 - alpha/16),
+                           s^2 * searchwidth * n / qchisq(df=df, alpha/16), length.out=steps)
+        
+        grid <- expand.grid(togrid)
+        
+        grid[, 3] <- n/s^2 * (mea - grid[, 1])^2 + n/(2 * s^4) * (s^2 - grid[, 2])^2 < rhs
+        
+        crFinal <- grid[grid[, 3]==TRUE, ]
+        ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
+                
+      }
+      
+      while((nrow(crFinal[crFinal$Var2==min(crFinal$Var2), ]) > 10 |
+              nrow(crFinal[crFinal$Var2==max(crFinal$Var2), ]) > 10) & Vsteps < 2001){
+        
+        Vsteps <- 2 * Vsteps
+        
+        togrid <- list()
+        togrid[[1]] <- seq(mea - searchwidth * qnorm(1 - alpha/16) * s / sqrt(n),
+                           mea + searchwidth * qnorm(1 - alpha/16) * s / sqrt(n), length.out=Msteps)
+        togrid[[2]] <- seq(s^2 * 1/searchwidth * n / qchisq(df=df, 1 - alpha/16),
+                           s^2 * searchwidth * n / qchisq(df=df, alpha/16), length.out=Vsteps)
+        
+        grid <- expand.grid(togrid)
+        
+        grid[, 3] <- n/s^2 * (mea - grid[, 1])^2 + n/(2 * s^4) * (s^2 - grid[, 2])^2 < rhs
+        
+        crFinal <- grid[grid[, 3]==TRUE, ]
+        ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
+                
+      }
+      
     }
     
     if(method=="pluginF"){
       
-      grid[, 3] <- n/s^2 * (mea - grid[, 1])^2 + n/(2 * s^4) * (s^2 - grid[, 2])^2 < qf(1 - alpha, df1=2, df2=n - 2)
+      rhs <- qf(1 - alpha, df1=2, df2=n - 2)
+      grid[, 3] <- n/s^2 * (mea - grid[, 1])^2 + n/(2 * s^4) * (s^2 - grid[, 2])^2 < rhs
       
       crFinal <- grid[grid[, 3]==TRUE, ]
       ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
@@ -150,11 +237,54 @@ csetMV <- function(dat, n, method, alpha=0.1, scale="var", steps=400){
         
       }
       
+      Msteps <- Vsteps <- steps
+      
+      while((nrow(crFinal[crFinal$Var1==min(crFinal$Var1), ]) > 5 |
+              nrow(crFinal[crFinal$Var1==max(crFinal$Var1), ]) > 5) & Msteps < 2001){
+        
+        Msteps <- 2 * Msteps
+        
+        togrid <- list()
+        togrid[[1]] <- seq(mea - searchwidth * qnorm(1 - alpha/16) * s / sqrt(n),
+                           mea + searchwidth * qnorm(1 - alpha/16) * s / sqrt(n), length.out=Msteps)
+        togrid[[2]] <- seq(s^2 * 1/searchwidth * n / qchisq(df=df, 1 - alpha/16),
+                           s^2 * searchwidth * n / qchisq(df=df, alpha/16), length.out=steps)
+        
+        grid <- expand.grid(togrid)
+        
+        grid[, 3] <- n/s^2 * (mea - grid[, 1])^2 + n/(2 * s^4) * (s^2 - grid[, 2])^2 < rhs
+        
+        crFinal <- grid[grid[, 3]==TRUE, ]
+        ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
+                
+      }
+      
+      while((nrow(crFinal[crFinal$Var2==min(crFinal$Var2), ]) > 10 |
+              nrow(crFinal[crFinal$Var2==max(crFinal$Var2), ]) > 10) & Vsteps < 2001){
+        
+        Vsteps <- 2 * Vsteps
+        
+        togrid <- list()
+        togrid[[1]] <- seq(mea - searchwidth * qnorm(1 - alpha/16) * s / sqrt(n),
+                           mea + searchwidth * qnorm(1 - alpha/16) * s / sqrt(n), length.out=Msteps)
+        togrid[[2]] <- seq(s^2 * 1/searchwidth * n / qchisq(df=df, 1 - alpha/16),
+                           s^2 * searchwidth * n / qchisq(df=df, alpha/16), length.out=Vsteps)
+        
+        grid <- expand.grid(togrid)
+        
+        grid[, 3] <- n/s^2 * (mea - grid[, 1])^2 + n/(2 * s^4) * (s^2 - grid[, 2])^2 < rhs
+        
+        crFinal <- grid[grid[, 3]==TRUE, ]
+        ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
+                
+      }
+      
     }
     
     if(method=="lrt"){
       
-      grid[, 3] <- n * log(grid[, 2] / s^2) + n * s^2 / grid[, 2] + n * (mea - grid[, 1])^2 / grid[, 2] - n < qchisq(1 - alpha, df=2)
+      rhs <- qchisq(1 - alpha, df=2)
+      grid[, 3] <- n * log(grid[, 2] / s^2) + n * s^2 / grid[, 2] + n * (mea - grid[, 1])^2 / grid[, 2] - n < rhs
       
       crFinal <- grid[grid[, 3]==TRUE, ]
       ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
@@ -170,7 +300,7 @@ csetMV <- function(dat, n, method, alpha=0.1, scale="var", steps=400){
         
         grid <- expand.grid(togrid)
         
-        grid[, 3] <- n * log(grid[, 2] / s^2) + n * s^2 / grid[, 2] + n * (mea - grid[, 1])^2 / grid[, 2] - n < qchisq(1 - alpha, df=2)
+        grid[, 3] <- n * log(grid[, 2] / s^2) + n * s^2 / grid[, 2] + n * (mea - grid[, 1])^2 / grid[, 2] - n < rhs
         
         crFinal <- grid[grid[, 3]==TRUE, ]
         ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
@@ -179,6 +309,48 @@ csetMV <- function(dat, n, method, alpha=0.1, scale="var", steps=400){
         
       }
       
+      Msteps <- Vsteps <- steps
+      
+      while((nrow(crFinal[crFinal$Var1==min(crFinal$Var1), ]) > 5 |
+              nrow(crFinal[crFinal$Var1==max(crFinal$Var1), ]) > 5) & Msteps < 2001){
+        
+        Msteps <- 2 * Msteps
+        
+        togrid <- list()
+        togrid[[1]] <- seq(mea - searchwidth * qnorm(1 - alpha/16) * s / sqrt(n),
+                           mea + searchwidth * qnorm(1 - alpha/16) * s / sqrt(n), length.out=Msteps)
+        togrid[[2]] <- seq(s^2 * 1/searchwidth * n / qchisq(df=df, 1 - alpha/16),
+                           s^2 * searchwidth * n / qchisq(df=df, alpha/16), length.out=steps)
+        
+        grid <- expand.grid(togrid)
+        
+        grid[, 3] <- n * log(grid[, 2] / s^2) + n * s^2 / grid[, 2] + n * (mea - grid[, 1])^2 / grid[, 2] - n < qchisq(1 - alpha, df=2)
+        
+        crFinal <- grid[grid[, 3]==TRUE, ]
+        ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
+                
+      }
+      
+      while((nrow(crFinal[crFinal$Var2==min(crFinal$Var2), ]) > 10 |
+              nrow(crFinal[crFinal$Var2==max(crFinal$Var2), ]) > 10) & Vsteps < 2001){
+        
+        Vsteps <- 2 * Vsteps
+        
+        togrid <- list()
+        togrid[[1]] <- seq(mea - searchwidth * qnorm(1 - alpha/16) * s / sqrt(n),
+                           mea + searchwidth * qnorm(1 - alpha/16) * s / sqrt(n), length.out=Msteps)
+        togrid[[2]] <- seq(s^2 * 1/searchwidth * n / qchisq(df=df, 1 - alpha/16),
+                           s^2 * searchwidth * n / qchisq(df=df, alpha/16), length.out=Vsteps)
+        
+        grid <- expand.grid(togrid)
+        
+        grid[, 3] <- n * log(grid[, 2] / s^2) + n * s^2 / grid[, 2] + n * (mea - grid[, 1])^2 / grid[, 2] - n < rhs
+        
+        crFinal <- grid[grid[, 3]==TRUE, ]
+        ciFinal <- t(apply(crFinal[, -3], 2, range, na.rm=TRUE))
+                
+      }
+      
     }
     
   }
