@@ -1,6 +1,6 @@
 print.JOC <- function(x, digits=max(3, getOption("digits") - 4), ...){
   
-  if(x$method %in% c("expanded", "fixseq", "tost")){
+  if(x$method %in% c("expanded", "fix.seq", "tost")){
     
     cat(paste("Parameter estimates and ", 100 * (1 - x$alpha), "% simultaneous confidence intervals:\n\n", sep=""))
     
@@ -20,7 +20,7 @@ print.JOC <- function(x, digits=max(3, getOption("digits") - 4), ...){
 
 summary.JOC <- function(object, digits=max(3, getOption("digits") - 4), ...){
   
-  if(object$method %in% c("expanded", "fixseq", "tost")){
+  if(object$method %in% c("expanded", "fix.seq", "tost")){
     
     cat(paste("Parameter estimates and ", 100 * (1 - object$alpha), "% simultaneous confidence intervals:\n\n", sep=""))
     
@@ -39,7 +39,7 @@ summary.JOC <- function(object, digits=max(3, getOption("digits") - 4), ...){
 }
 
 plot.JOC <- function(x, equi=log(c(0.8, 1.25)), axnames=NULL, main=NULL, xlim=log(c(0.77, 1.3)),
-                     ylim=log(c(0.77, 1.3)), col="black", ...){
+                     ylim=log(c(0.77, 1.3)), col="black", convexify=FALSE, ...){
   
   if(nrow(x$ci)!=2){
     stop("Plotting only allowed for regions or intervals in 2 dimensions.")
@@ -60,13 +60,20 @@ plot.JOC <- function(x, equi=log(c(0.8, 1.25)), axnames=NULL, main=NULL, xlim=lo
     }
     rect(equi[1], equi[1], equi[2], equi[2], col="gray95", border=NA)
   }
-  if(x$method %in% c("limacon.asy", "limacon.fin", "tseng", "tseng.brown")){
-    points(x$cr, pch=20, col=col, cex=0.5)
+  if(x$method %in% c("limacon.asy", "limacon.fin")){
+    if(convexify==FALSE){
+      #points(x$cr, pch=20, col=col, cex=0.5)
+      tsp <- TSP(dist(x$cr))
+      tour <- solve_TSP(tsp, method='farthest')
+      polygon(x$cr[tour, ], col=NULL, border=col, lwd=2)
+    }else{
+      polygon(x$cr[chull(x$cr), -3], col=NULL, border=col, lwd=2)
+    }
   }
-  if(x$method %in% c("emp.bayes", "hotelling", "standard.cor", "standard.ind")){
+  if(x$method %in% c("boot.kern", "emp.bayes", "hotelling", "standard.cor", "standard.ind", "tseng", "tseng.brown")){
     polygon(x$cr[chull(x$cr), -3], col=NULL, border=col, lwd=2)
   }
-  if(x$method %in% c("expanded", "fixseq", "tost")){
+  if(x$method %in% c("expanded", "fix.seq", "tost")){
     segments(x0=x$ci[1], x1=x$ci[3], y0=x$est[2], y1=x$est[2], lwd=2, col=col)
     segments(y0=x$ci[2], y1=x$ci[4], x0=x$est[1], x1=x$est[1], lwd=2, col=col)
   }
